const { isNull } = require('lodash');
const db = require('../models');
const Utils = require('../utils');

const { uuid } = require('uuidv4');
const sendPdf = require

const Sequelize = db.Sequelize;
const Op = Sequelize.Op;
const BookingsService = require('../../services/bookings');


module.exports = class PujaModelsDBApi {
  static async create(data, options) {
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = await db.sequelize.transaction();
    const puja_models = await db.puja_models.create(
      {
        puja_id: data.puja_id || null,
        duration_hrs: data.duration_hrs || null,
        pujari_cost: data.pujari_cost || null,
        no_of_pujaris: data.no_of_pujaris || null,
        model_selling_price: data.model_selling_price || null,
        advance_amount: data.advance_amount || null,
        is_popular_model: data.is_popular_model === null ? null : data.is_popular_model,
        indicative_cost_price: data.indicative_cost_price || null,
        name: data.name || null,
        samagri_cost: data.samagri_cost || null,
        service_fee: data.service_fee || null,
        importHash: data.importHash || null,
        createdById: currentUser.id,
        updatedById: currentUser.id,
      },
      { transaction },
    );

    // procedure mappings
    var procedureData = data.procedure;

    const procedureDataToDB = procedureData.map((procedure) => {
      return {
        ...procedure,
        id: uuid(),
        model_id: puja_models.dataValues.id,
      };
    });

    var puja_model_procedure_mappings;
    const validateProcedure = procedureDataToDB.every(
      (procedure) =>
        procedure.procedure_id !== undefined ||
        (null && procedure.order !== undefined) ||
        null,
    );

    if (validateProcedure) {
      puja_model_procedure_mappings =
        await db.puja_model_procedure_mappings.bulkCreate(procedureDataToDB, {
          transaction,
        });
      await transaction.commit();
    } else {
      await transaction.rollback();
      puja_model_procedure_mappings = false;
      return { message: 'Procedure Id & order  is required', statusCode: 500 };
    }

    return {
      statusCode: 200,
      puja_models: puja_models,
      puja_model_procedure_mappings: puja_model_procedure_mappings,
    };
  }

  static async findAll(filter, options) {

    var limit = filter.limit || 0;
    var offset = 0;
    const currentPage = +filter.page;

    offset = currentPage * limit;

    var orderBy = null;

    const transaction = (options && options.transaction) || undefined;
    let where = {};
    let include = [db.puja_model_procedure_mappings];

    if (filter) {
      if (filter.id) {
        where = {
          ...where,
          ['id']: Utils.uuid(filter.id),
        };
      }

      if (filter.name) {
        where = {
          ...where,
          [Op.and]: Utils.ilike('pujas', 'name', filter.name),
        };
      }

      if (filter.description) {
        where = {
          ...where,
          [Op.and]: Utils.ilike('pujas', 'description', filter.description),
        };
      }

      if (filter.duration_hrsRange) {
        const [start, end] = filter.duration_hrsRange;

        if (start !== undefined && start !== null && start !== '') {
          where = {
            ...where,
            duration_hrs: {
              ...where.duration_hrs,
              [Op.gte]: start,
            },
          };
        }

        if (end !== undefined && end !== null && end !== '') {
          where = {
            ...where,
            duration_hrs: {
              ...where.duration_hrs,
              [Op.lte]: end,
            },
          };
        }
      }

      if (
        filter.active === true ||
        filter.active === 'true' ||
        filter.active === false ||
        filter.active === 'false'
      ) {
        where = {
          ...where,
          active: filter.active === true || filter.active === 'true',
        };
      }

      if (filter.language) {
        where = {
          ...where,
          language: filter.language,
        };
      }

      if (filter.createdAtRange) {
        const [start, end] = filter.createdAtRange;

        if (start !== undefined && start !== null && start !== '') {
          where = {
            ...where,
            ['createdAt']: {
              ...where.createdAt,
              [Op.gte]: start,
            },
          };
        }

        if (end !== undefined && end !== null && end !== '') {
          where = {
            ...where,
            ['createdAt']: {
              ...where.createdAt,
              [Op.lte]: end,
            },
          };
        }
      }
    }

    let { rows, count } = await db.puja_models.findAndCountAll({
      where,
      include,
      distinct: true,
      limit: limit ? Number(limit) : undefined,
      offset: offset ? Number(offset) : undefined,
      // order:
      //   filter.field && filter.sort
      //     ? [[filter.field, filter.sort]]
      //     : [['createdAt', 'desc']],
      order: [["updatedAt", "desc"]],
      transaction,
    });

    //    rows = await this._fillWithRelationsAndFilesForRows(
    //      rows,
    //      options,
    //    );

    return { rows, count };
  }

  static async findBy(where, options) {
    const transaction = (options && options.transaction) || undefined;

    let include = [db.puja_model_procedure_mappings];
    const puja_models = await db.puja_models.findOne(
      { where, include },
      { transaction },
    );

    if (!puja_models) {
      return puja_models;
    }

    const output = puja_models.get({ plain: true });

    return output;
  }

  static async update(data, id, options) {
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const puja_models = await db.puja_models.findByPk(id, {
      transaction,
    });

    const payloadData = {
      puja_id: data.puja_id || null,
      duration_hrs: data.duration_hrs || null,
      pujari_cost: data.pujari_cost || null,
      no_of_pujaris: data.no_of_pujaris || null,
      model_selling_price: data.model_selling_price || null,
      advance_amount: data.advance_amount || null,
      is_popular_model: data.is_popular_model == 0 ? 0 : data.is_popular_model || null,
      indicative_cost_price: data.indicative_cost_price || null,
      name: data.name || null,
      samagri_cost: data.samagri_cost || null,
      service_fee: data.service_fee || null,
      updatedById: currentUser.id,
    };
    await puja_models.update(
      payloadData,
      { transaction },
    );

    // procedure mappings update

    const newProcedure = data.procedure.filter(
      (procedure) => procedure.id === undefined,
    );

    const updatingProcedure = data.procedure.filter(
      (procedure) => procedure.id !== undefined,
    );

    const deleteSamagriResponse =
      await db.puja_model_procedure_mappings.destroy({
        where: { model_id: id },
        force: true,
      });

    const procedureAddToDB = data.procedure.map((procedure) => {
      return {
        ...procedure,
        id: uuid(),
        model_id: id,
      };
    });

    const procedureAddedResponse =
      await db.puja_model_procedure_mappings.bulkCreate(procedureAddToDB, {
        transaction,
      });

    return puja_models;
  }

  static async remove(id, options) {
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const puja_models = await db.puja_models.findByPk(id, options);

    await db.puja_models.destroy({
      where: { id },
      force: true,
    });

    await db.puja_model_procedure_mappings.destroy({
      where: { model_id: id },
      force: true,
    });

    return puja_models;
  }

  static async totalSamagri(data, options) {
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;
    let where = {};
    let include = [];

    const [samagriMRP] = await db.sequelize.query(`SELECT SUM(map.no_of_standard_qty * samagri.customer_mrp) as MRP FROM puja_samagri_mappings as map, samagri as samagri where 
 map.samagri_id=samagri.id and map.isProvidedBy='karishye' and map.puja_id=${data.id} and map.samagri_type='default'`, { transaction });

    const [samagriTotalPrice] = await db.sequelize.query(`SELECT SUM(map.no_of_standard_qty * samagri.cost_price) as TOTALPRICE FROM puja_samagri_mappings as map, 
    samagri as samagri where map.samagri_id=samagri.id and map.isProvidedBy='karishye' and map.puja_id=${data.id} and map.samagri_type='default'`, { transaction });

    // // puja samagri mappings //
    // const pujaSamagriMappings = await db.puja_samagri_mappings.findAll({
    //   where: { puja_id: data.id },
    //   include,
    //   distinct: true,
    //   transaction,
    // });

    // const pujaSamagriMappingsFilter = pujaSamagriMappings.map((samagri) => {
    //   let res = samagri.dataValues;
    //   return res;
    // });

    // // samagri //

    // const pujaSamagriIds = pujaSamagriMappingsFilter.map((samagri) => {
    //   return samagri.samagri_id;
    // });

    // const samagri = await db.samagri.findAll({
    //   where: { id: pujaSamagriIds },
    //   include,
    //   distinct: true,
    //   transaction,
    // });

    // const filteredSamagri = samagri.map((samagri) => {
    //   return samagri.dataValues;
    // });


    // const price = () => {
    //   var totalMRP = 0;
    //   var totalCost = 0;
    //   for (var i = 0; i < filteredSamagri.length; i++) {
    //     if (pujaSamagriMappings[i].samagri_type == 'default') {
    //       totalMRP =
    //         totalMRP +
    //         pujaSamagriMappingsFilter[i].no_of_standard_qty *
    //         filteredSamagri[i].customer_mrp;
    //       totalCost = totalCost + pujaSamagriMappingsFilter[i].no_of_standard_qty *
    //         filteredSamagri[i].cost_price;
    //     }
    //   }

    //   return { totalMRP, totalCost };
    // };
    return { cost: { totalMRP: Number(samagriMRP[0]["MRP"]), totalCost: Number(samagriTotalPrice[0]["TOTALPRICE"]) } };
  }
};
